
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/nodemask.h>
#include <linux/platform_device.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/irq.h>
#include <asm/mach/keypad.h>
#include <asm/arch/memory.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>

#include "iomux.h"

#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/cfi.h>
#include <asm/mach/flash.h>
#include <asm/arch/mtd.h>
#include <asm/arch/clock.h>

#include <asm/arch/i2c-armadillo5x0.h>

/*#include "mxc_nd.h"*/
#include "crm_regs.h"

#define FLASH_NOR_BASE_ADDR	(0xa0000000)
#define FLASH_NOR_SIZE		(0x04000000) /* Max: 64MB */


extern void mxc_map_io(void);
extern void mxc_init_irq(void);
extern void mxc_cpu_init(void) __init;
extern struct sys_timer mxc_timer;

extern void armadillo5x0_gpio_init(void) __init;
extern void mxc_cpu_common_init(void);


u32
get_machine_id(void)
{
	return 0;
}
EXPORT_SYMBOL(get_machine_id);

static struct resource armadillo5x0_smc911x_resources[] = {
	[0] = {
		.start	= 0xb2000000,
		.end	= 0xb2000000 + 0x02000000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= 64,//INT_GPIO1,
		.end	= 64,//INT_GPIO1,
		.flags	= IORESOURCE_IRQ,
	},
};
struct smsc911x_platform_config {
	unsigned int irq_polarity;
	unsigned int irq_type;
};
static struct smsc911x_platform_config smsc911x_config = {
	.irq_polarity	= 0,
	.irq_type	= 1,
};
static struct platform_device armadillo5x0_smc911x_device = {
	.name		= "smsc911x",
	.id		= 0,
	.dev		= {
		.platform_data = &smsc911x_config,
	},
	.num_resources	= ARRAY_SIZE(armadillo5x0_smc911x_resources),  
	.resource	= armadillo5x0_smc911x_resources,
};

static void
armadillo5x0_eth_init(void)
{
	platform_device_register(&armadillo5x0_smc911x_device);
}

#define MTD_PART(_name, _size, _offset, _mask_flags) \
	{                                            \
		.name		= _name,             \
		.size		= _size,             \
		.offset		= _offset,           \
		.mask_flags	= _mask_flags,       \
	}

static struct mtd_partition mtd_nor_partitions_8MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",    46*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};
static struct mtd_partition mtd_nor_partitions_16MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",   110*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};
static struct mtd_partition mtd_nor_partitions_32MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",   238*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};
static struct mtd_partition mtd_nor_partitions_64MB[] = {
	MTD_PART("nor.bootloader",   4* 32*1024, 0,
		 MTD_WRITEABLE /* force read-only */ ),
	MTD_PART("nor.kernel",      16*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.userland",   494*128*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nor.config",       1*128*1024, MTDPART_OFS_APPEND, 0),
};

static int
mtd_updata_partitions(struct map_info *map,
		      struct flash_platform_data *plat)
{
	struct cfi_private *cfi = map->fldrv_priv;

	switch (cfi->cfiq->DevSize) {
	case 23: /* 8MB */
		plat->parts	= mtd_nor_partitions_8MB;
		plat->nr_parts	= ARRAY_SIZE(mtd_nor_partitions_8MB);
		break;
	case 24: /* 16MB */
		break;
	case 25: /* 32MB */
		plat->parts	= mtd_nor_partitions_32MB;
		plat->nr_parts	= ARRAY_SIZE(mtd_nor_partitions_32MB);
		break;
	case 26: /* 64MB */
		plat->parts	= mtd_nor_partitions_64MB;
		plat->nr_parts	= ARRAY_SIZE(mtd_nor_partitions_64MB);
		break;
	default:
		printk("Not support flash-size.\n");
		return -1;
	}

	return 0;
}

static struct armadillo500_flash_private_data mtd_armadillo5x0_private = {
	.plat = {
		.map_name	= "cfi_probe",
		.width		= 2,
		.parts		= mtd_nor_partitions_16MB,
		.nr_parts	= ARRAY_SIZE(mtd_nor_partitions_16MB),
	},
	.updata_partitions = mtd_updata_partitions,
};

static struct resource mtd_armadillo5x0_nor_resource = {
	.start		= FLASH_NOR_BASE_ADDR,
	.end		= FLASH_NOR_BASE_ADDR + FLASH_NOR_SIZE - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device mtd_armadillo5x0_nor_device = {
	.name		= "armadillo5x0-nor",
	.id		= 0,
	.dev		= {
		.platform_data = &mtd_armadillo5x0_private,
	},
	.num_resources	= 1,
	.resource	= &mtd_armadillo5x0_nor_resource,
};

static struct mtd_partition mtd_armadillo5x0_nand_partitions[] = {
	MTD_PART("nand.ipl",           1* 128*1024, 0, 0),
	MTD_PART("nand.kernel",        4*1024*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nand.userland",     22*1024*1024, MTDPART_OFS_APPEND, 0),
	MTD_PART("nand.free",     MTDPART_SIZ_FULL, MTDPART_OFS_APPEND, 0),
};

static struct flash_platform_data mtd_armadillo5x0_nand_data = {
	.parts = mtd_armadillo5x0_nand_partitions,
	.nr_parts = ARRAY_SIZE(mtd_armadillo5x0_nand_partitions),
	.width = 1,
};

static struct platform_device mtd_armadillo5x0_nand_device = {
	.name = "mxc_nand_flash",
	.id = 0,
	.dev = {
		.platform_data = &mtd_armadillo5x0_nand_data,
	},
};

static void
armadillo5x0_mtd_init(void)
{
	/* NOR */
	(void)platform_device_register(&mtd_armadillo5x0_nor_device);

	/* NAND */
	if (__raw_readl(MXC_CCM_RCSR) & MXC_CCM_RCSR_NF16B) {
		mtd_armadillo5x0_nand_data.width = 2;
	}
	(void)platform_device_register(&mtd_armadillo5x0_nand_device);
}

static struct platform_device armadillo5x0_fb_device = {
	.name	= "mxc_sdc_fb",
	.id	= 0,
	.dev	= {
		.platform_data		= "TV-VGA",
		.coherent_dma_mask	= 0xFFFFFFFF,
	},
};

static void
armadillo5x0_fb_init(void)
{
	platform_device_register(&armadillo5x0_fb_device);
}

static struct resource i2c1_resources[] = {
	[0] = {
		.start	= I2C_BASE_ADDR,
		.end	= I2C_BASE_ADDR + 0x4000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= INT_I2C,
		.end	= INT_I2C,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct i2c_armadillo5x0_private i2c1_private_data = {
	.name	= "I2C1",
	.clkdiv	= 960,
};

static struct resource i2c2_resources[] = {
	[0] = {
		.start	= I2C2_BASE_ADDR,
		.end	= I2C2_BASE_ADDR + 0x4000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= INT_I2C2,
		.end	= INT_I2C2,
		.flags	= IORESOURCE_IRQ,
	},
};

static struct i2c_armadillo5x0_private i2c2_private_data = {
	.name	= "I2C2",
	.clkdiv	= 960,
};

static struct platform_device armadillo5x0_i2c_devices[] = {
	{
		.name		= "i2c_armadillo5x0",
		.id		= 0,
		.dev		= {
			.platform_data	= &i2c1_private_data,
		},
		.num_resources	= ARRAY_SIZE(i2c1_resources),
		.resource	= i2c1_resources,
	},
	{
		.name		= "i2c_armadillo5x0",
		.id		= 1,
		.dev		= {
			.platform_data	= &i2c2_private_data,
		},
		.num_resources	= ARRAY_SIZE(i2c2_resources),
		.resource	= i2c2_resources,
	},
};

static void
armadillo5x0_i2c_init(void)
{
	int i;

	for (i=0; i<ARRAY_SIZE(armadillo5x0_i2c_devices); i++)
		platform_device_register(&armadillo5x0_i2c_devices[i]);
}

static struct platform_device armadillo5x0_led_device = {
	.name	= "armadillo5x0_led",
	.id	= 0,
};

static void
armadillo5x0_led_device_init(void)
{
	platform_device_register(&armadillo5x0_led_device);
}

static struct platform_device armadillo5x0_gpio_device = {
	.name = "armadillo5x0_gpio",
	.id = 0,
};

static void
armadillo5x0_gpio_device_init(void)
{
	platform_device_register(&armadillo5x0_gpio_device);
}

static struct platform_device armadillo5x0_tactsw_device = {
	.name = "armadillo5x0_tactsw",
	.id = 0,
};

static void
armadillo5x0_tactsw_device_init(void)
{
	platform_device_register(&armadillo5x0_tactsw_device);
}

extern int mxc_clocks_init(void);

static void __init
armadillo5x0_init(void)
{
	mxc_cpu_common_init();
	mxc_clocks_init();
	mxc_gpio_init();

	armadillo5x0_gpio_init();
	armadillo5x0_i2c_init();
	armadillo5x0_eth_init();
	armadillo5x0_mtd_init();
	armadillo5x0_fb_init();
	armadillo5x0_led_device_init();
	armadillo5x0_gpio_device_init();
	armadillo5x0_tactsw_device_init();
}

#if defined(CONFIG_MACH_ARMADILLO500)
MACHINE_START(ARMADILLO5X0, "Armadillo-500")
	/* Maintainer: Atmark Techno, Inc. */
	.phys_io	= AIPS1_BASE_ADDR,
	.io_pg_offst	= ((AIPS1_BASE_ADDR_VIRT) >> 18) & 0xfffc,
	.boot_params	= PHYS_OFFSET + 0x00000100,
	.map_io		= mxc_map_io,
	.init_irq	= mxc_init_irq,
	.timer		= &mxc_timer,
	.init_machine	= armadillo5x0_init,
MACHINE_END
#endif
