/*
 * arch/ppc/platforms/4xx/suzaku-v.h
 *
 * Include file that defines Atmark Techno's SUZAKU-V board
 *
 *  Copyright (C) 2005, 2007-2008	Atmark Techno, Inc.
 *
 * Yasushi SHOJI <yashi@atmark-techno.com>
 * Masahiro Hanada <hanada@atmark-techno.com>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of this
 * archive for more details.
 *
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/irq.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/ppc_sys.h>

#include <platforms/4xx/xparameters/xparameters.h>

/* Board specifications structures */
struct ppc_sys_spec *cur_ppc_sys_spec;
struct ppc_sys_spec ppc_sys_specs[] = {
	{
		/* Only one entry, always assume the same design */
#if defined(CONFIG_SUZAKU_SZ310)
		.ppc_sys_name	= "SUZAKU-V SZ310",
#elif defined(CONFIG_SUZAKU_SZ410)
		.ppc_sys_name	= "SUZAKU-V SZ410",
#endif
		.mask 		= 0x00000000,
		.value 		= 0x00000000,
		.num_devices	= NUM_PPC_SYS_DEVS,
		.device_list	= (enum ppc_sys_devices[])
		{
#if defined(CONFIG_SERIAL_8250)
			VIRTEX_UART,
#endif
#if defined(CONFIG_SERIAL_UARTLITE) || defined(CONFIG_SERIAL_UARTLITE_MODULE)
			UARTLITE_UART,
#endif
#if defined(CONFIG_XILINX_TEMAC) || defined(CONFIG_XILINX_TEMAC_MODULE) || \
    defined(CONFIG_XILINX_OLD_TEMAC) || defined(CONFIG_XILINX_OLD_TEMAC_MODULE) || \
    defined(CONFIG_PICO_TEMAC) || defined(CONFIG_PICO_TEMAC_MODULE)
			VIRTEX_TEMAC_0,
#endif
		},
	},
};

unsigned char *suzaku_sysreg;
unsigned char *suzaku_led_gpio;
EXPORT_SYMBOL(suzaku_sysreg);
EXPORT_SYMBOL(suzaku_led_gpio);

static void __init sv_map_io(void)
{
	ppc4xx_map_io();

	suzaku_sysreg = ioremap_nocache(SUZAKU_SYSREG_BASE_ADDR,
					SUZAKU_SYSREG_SIZE);
	suzaku_led_gpio = ioremap_nocache(SUZAKU_LED_GPIO_BASE_ADDR,
					  SUZAKU_LED_GPIO_SIZE);
}

static void __init sv_setup_arch(void)
{
	ppc4xx_setup_arch();
#if defined(CONFIG_SUZAKU_SZ310)
	printk(KERN_INFO "Atmark Techno SUZAKU-V SZ310\n");
#elif defined(CONFIG_SUZAKU_SZ410)
	printk(KERN_INFO "Atmark Techno SUZAKU-V SZ410\n");
#endif
}

static void __init sv_init_irq(void)
{
	ppc4xx_init_IRQ();
}

void __init platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
			  unsigned long r6, unsigned long r7)
{
	console_verbose();

	ppc4xx_init(r3, r4, r5, r6, r7);

	identify_ppc_sys_by_id(mfspr(SPRN_PVR));

	ppc_md.setup_arch = sv_setup_arch;
	ppc_md.setup_io_mappings = sv_map_io;
	ppc_md.init_IRQ = sv_init_irq;
}

void memset_io(volatile void __iomem *dst, int c, __kernel_size_t n)
{
	volatile void __iomem *d = dst;

	while (n--) {
		writeb(c, d);
		d++;
	}
}

void memcpy_fromio(void *dst, const volatile void __iomem *src, __kernel_size_t n)
{
	char *d = dst;

	while (n--) {
		char tmp = readb(src);
		*d++ = tmp;
		src++;
	}
}

void memcpy_toio(volatile void __iomem *dst, const void *src, __kernel_size_t n)
{
	const char *s = src;
	volatile void __iomem *d = dst;

	while (n--) {
		char tmp = *s++;
		writeb(tmp, d);
		d++;
	}
}

#if defined(CONFIG_SMC91X) || defined(CONFIG_SMC91X_MODULE)

#define SMC91111_BASE_ADDR      0xf0e00300
#define SMC91111_REG_SIZE       16
#define SMC91111_IRQ            1

static struct resource smc91x_resources[] = {
	[0] = {
		.start = SMC91111_BASE_ADDR,
		.end   = SMC91111_BASE_ADDR + SMC91111_REG_SIZE - 1,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = SMC91111_IRQ,
		.end   = SMC91111_IRQ,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device smc91x_device = {
	.name	       = "smc91x",
	.id	       = -1,
	.num_resources = ARRAY_SIZE(smc91x_resources),
	.resource      = smc91x_resources,
};

#endif

#if defined(CONFIG_SUZAKU_SID) || defined(CONFIG_SUZAKU_SID_MODULE)

static struct resource sid_resources[] = {
	[0] = {
		.start = XPAR_SID_CNTLR_MEM0_BASEADDR,
		.end   = XPAR_SID_CNTLR_MEM0_HIGHADDR,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = XPAR_INTC_SYSTEM_SID_CNTLR_SID_INTR_INTR,
		.end   = XPAR_INTC_SYSTEM_SID_CNTLR_SID_INTR_INTR,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device sid_device = {
	.name	       = "sid",
	.id	       = -1,
	.num_resources = ARRAY_SIZE(sid_resources),
	.resource      = sid_resources,
};

#endif

#if defined(CONFIG_SUZAKU_SIV) || defined(CONFIG_SUZAKU_SIV_MODULE)

static struct resource siv_resources[] = {
	[0] = {
		.start = XPAR_SIV_CNTLR_MEM0_BASEADDR,
		.end   = XPAR_SIV_CNTLR_MEM0_HIGHADDR,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = XPAR_INTC_SYSTEM_SIV_CNTLR_SIV_INTR_INTR,
		.end   = XPAR_INTC_SYSTEM_SIV_CNTLR_SIV_INTR_INTR,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device siv_device = {
	.name	       = "siv",
	.id	       = -1,
	.num_resources = ARRAY_SIZE(siv_resources),
	.resource      = siv_resources,
};

#endif

#if defined(CONFIG_SUZAKU_SIA) || defined(CONFIG_SUZAKU_SIA_MODULE)

static struct resource sia_resources[] = {
	[0] = {
		.start = XPAR_SIA_CNTLR_MEM0_BASEADDR,
		.end   = XPAR_SIA_CNTLR_MEM0_HIGHADDR,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = XPAR_INTC_SYSTEM_SIA_CNTLR_SIA_INTR_INTR,
		.end   = XPAR_INTC_SYSTEM_SIA_CNTLR_SIA_INTR_INTR,
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device sia_device = {
	.name	       = "sia",
	.id	       = -1,
	.num_resources = ARRAY_SIZE(sia_resources),
	.resource      = sia_resources,
};

#endif

#if defined(CONFIG_I2C_SUZAKU) || defined(CONFIG_I2C_SUZAKU_MODULE)

static struct resource i2c_suzaku_resources[] = {
	[0] = {
		.start = XPAR_I2C_BASEADDR,
		.end   = XPAR_I2C_HIGHADDR,
		.flags = IORESOURCE_MEM,
	},
};

static struct platform_device i2c_suzaku_device = {
	.name	       = "i2c-suzaku",
	.id	       = -1,
	.num_resources = ARRAY_SIZE(i2c_suzaku_resources),
	.resource      = i2c_suzaku_resources,
};

#endif

static struct platform_device *sv_devices[] __initdata = {
#if defined(CONFIG_SMC91X) || defined(CONFIG_SMC91X_MODULE)
	&smc91x_device,
#endif
#if defined(CONFIG_SUZAKU_SID) || defined(CONFIG_SUZAKU_SID_MODULE)
	&sid_device,
#endif
#if defined(CONFIG_SUZAKU_SIV) || defined(CONFIG_SUZAKU_SIV_MODULE)
	&siv_device,
#endif
#if defined(CONFIG_SUZAKU_SIA) || defined(CONFIG_SUZAKU_SIA_MODULE)
	&sia_device,
#endif
#if defined(CONFIG_I2C_SUZAKU) || defined(CONFIG_I2C_SUZAKU_MODULE)
	&i2c_suzaku_device,
#endif
};

static int __init sv_device_init(void)
{
	return platform_add_devices(sv_devices, ARRAY_SIZE(sv_devices));
}

device_initcall(sv_device_init);
