/*
 * drivers/mtd/maps/attached_romfs.c -- Generic Attached ROMFS Map Driver
 *
 *  Copyright (C) 2004-2005	Atmark Techno <yashi@atmark-techno.com>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of this
 * archive for more details.
 *
 * This code is derived work of drivers/mtd/maps/uclinux.c
 *	(C) Copyright 2002, Greg Ungerer (gerg@snapgear.com)
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/root_dev.h>
#include <linux/romfs_fs.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <asm/sections.h>

static struct map_info romfs_map = {
	.name = "RAM",
};

static struct mtd_info *romfs_mtd;

static struct mtd_partition romfs_partitions[] = {
	{ .name = "ROMfs" }
};

static int __init attached_romfs_init(void)
{
	int ret = 0;
	int size;
	extern char *klimit;

	size = romfs_get_size((struct romfs_super_block*)(__bss_stop));
	if (romfs_map.size < 0) {
		printk("Attached ROMFS: unable to find romfs at the end of kernel (%p)\n", __bss_stop);
		ret = -ENXIO;
		goto end;
	}

	/* we _know_ this is located in ram but the map_info->virt is
	 * decleared with __iomem.  we'll cast __bss_stop to void
	 * __iomem * to kill sparse warning.  or should I use
	 * __force? --yashi */
	romfs_map.virt	    = (void __iomem *)__bss_stop;
	romfs_map.size	    = PAGE_ALIGN(size);
	romfs_map.phys	    = __pa(__bss_stop);
	romfs_map.bankwidth = 4;

	printk("Attached ROMFS: RAM probe address=%#lx size=%#lx\n",
	       (unsigned long)romfs_map.virt, romfs_map.size);

	romfs_mtd = do_map_probe("map_ram", &romfs_map);
	if (romfs_mtd) {
		romfs_mtd->owner = THIS_MODULE;
		romfs_mtd->priv	 = &romfs_map;
		add_mtd_device(romfs_mtd);
		printk("Attached ROMFS: set %s to be root filesystem\n", romfs_partitions[0].name);
		ROOT_DEV = MKDEV(MTD_BLOCK_MAJOR, romfs_mtd->index);
	} else {
		printk("Attached ROMFS: failed to find a mapping\n");
		ret = -ENXIO;
	}
end:
	return ret;
}

static void __exit attached_romfs_cleanup(void)
{
	if (romfs_mtd) {
		del_mtd_device(romfs_mtd);
		map_destroy(romfs_mtd);
		romfs_mtd = NULL;
	}
}

module_init(attached_romfs_init);
module_exit(attached_romfs_cleanup);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Yasushi SHOJI <yashi@atmark-techno.com");
MODULE_DESCRIPTION("Generic Attached ROMFS Map Driver");
