/*
 * Copyright 2005-2006 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef __ASM_ARCH_MXC_BOARD_MX31ADS_H__
#define __ASM_ARCH_MXC_BOARD_MX31ADS_H__

/*
 * Include Files
 */
#include <asm/arch/board.h>

/* Start of physical RAM */
#define PHYS_OFFSET	        UL(0x80000000)

/* Size of contiguous memory for DMA and other h/w blocks */
#define CONSISTENT_DMA_SIZE	SZ_8M

/* I2C configuration */
/*!
 * This defines the number of I2C modules in the MXC platform
 * Defined as 1, as MC13783 on ADS uses the other pins
 */
#define I2C_NR                  1
/*!
 * This define specifies the frequency divider value to be written into
 * the I2C \b IFDR register.
 */
#define I2C1_FRQ_DIV            0x17

/*!
 * @name MXC UART EVB board level configurations
 */
/*! @{ */
/*!
 * Specify the max baudrate for the MXC UARTs for your board, do not specify a max
 * baudrate greater than 1500000. This is used while specifying the UART Power
 * management constraints.
 */
#define MAX_UART_BAUDRATE       1500000
/*!
 * Specifies if the Irda transmit path is inverting
 */
#define MXC_IRDA_TX_INV         0
/*!
 * Specifies if the Irda receive path is inverting
 */
#define MXC_IRDA_RX_INV         0
/* UART 1 configuration */
/*!
 * This define specifies if the UART port is configured to be in DTE or
 * DCE mode. There exists a define like this for each UART port. Valid
 * values that can be used are \b MODE_DTE or \b MODE_DCE.
 */
#define UART1_MODE              MODE_DTE
/*!
 * This define specifies if the UART is to be used for IRDA. There exists a
 * define like this for each UART port. Valid values that can be used are
 * \b IRDA or \b NO_IRDA.
 */
#define UART1_IR                NO_IRDA
/*!
 * This define is used to enable or disable a particular UART port. If
 * disabled, the UART will not be registered in the file system and the user
 * will not be able to access it. There exists a define like this for each UART
 * port. Specify a value of 1 to enable the UART and 0 to disable it.
 */
#define UART1_ENABLED           1
/*! @} */
/* UART 2 configuration */
#define UART2_MODE              MODE_DTE
#define UART2_IR                NO_IRDA
#define UART2_ENABLED           1
/* UART 3 configuration */
#define UART3_MODE              MODE_DTE
#define UART3_IR                NO_IRDA
#define UART3_ENABLED           0
/* UART 4 configuration */
#define UART4_MODE              MODE_DTE
#define UART4_IR                NO_IRDA
#define UART4_ENABLED           0	/* Disable UART 4 as its pins are shared with ATA */
/* UART 5 configuration */
#define UART5_MODE              MODE_DTE
#define UART5_IR                NO_IRDA
#define UART5_ENABLED           0

#if 0
#define MXC_LL_EXTUART_PADDR	(CS4_BASE_ADDR + 0x10000)
#define MXC_LL_EXTUART_VADDR	CS4_IO_ADDRESS(MXC_LL_EXTUART_PADDR)
#undef  MXC_LL_EXTUART_16BIT_BUS

#define MXC_LL_UART_PADDR	UART1_BASE_ADDR
#define MXC_LL_UART_VADDR	AIPS1_IO_ADDRESS(UART1_BASE_ADDR)
#else
#define MXC_LL_EXTUART_PADDR	(Error_using_CPLD)
#define MXC_LL_EXTUART_VADDR	(Error_using_CPLD)

#define MXC_LL_UART_PADDR	UART1_BASE_ADDR
#define MXC_LL_UART_VADDR	AIPS1_IO_ADDRESS(UART1_BASE_ADDR)
#endif

/*!
 * @name Memory Size parameters
 */
/*! @{ */
/*!
 * Size of SDRAM memory
 */
#define SDRAM_MEM_SIZE          SZ_128M
/*!
 * Size of MBX buffer memory
 */
#define MXC_MBX_MEM_SIZE        SZ_16M
/*!
 * Size of memory available to kernel
 */
#define MEM_SIZE                (SDRAM_MEM_SIZE - MXC_MBX_MEM_SIZE)
/*! @} */

/*!
 * @name Keypad Configurations FIXME
 */
/*! @{ */
/*!
 * Maximum number of rows (0 to 7)
 */
#define MAXROW                  8
/*!
 * Maximum number of columns (0 to 7)
 */
#define MAXCOL                  8
/*! @} */

#define PBC_BASE_ADDRESS        (Error_using_CPLD)
#define PBC_BSTAT               (Error_using_CPLD)
#define PBC_BCTRL1_SET          (Error_using_CPLD)
#define PBC_BCTRL1_CLEAR        (Error_using_CPLD)
#define PBC_BCTRL2_SET          (Error_using_CPLD)
#define PBC_BCTRL2_CLEAR        (Error_using_CPLD)
#define PBC_BCTRL3_SET          (Error_using_CPLD)
#define PBC_BCTRL3_CLEAR        (Error_using_CPLD)
#define PBC_BCTRL4_SET          (Error_using_CPLD)
#define PBC_BCTRL4_CLEAR        (Error_using_CPLD)
#define PBC_BSTAT1              (Error_using_CPLD)
#define PBC_INTSTATUS           (Error_using_CPLD)
#define PBC_INTCURR_STATUS      (Error_using_CPLD)
#define PBC_INTMASK_SET         (Error_using_CPLD)
#define PBC_INTMASK_CLEAR       (Error_using_CPLD)
#define PBC_SC16C652_UARTA      (Error_using_CPLD)
#define PBC_SC16C652_UARTB      (Error_using_CPLD)
#define PBC_CS8900A_IOBASE      (Error_using_CPLD)
#define PBC_CS8900A_MEMBASE     (Error_using_CPLD)
#define PBC_CS8900A_DMABASE     (Error_using_CPLD)
#define PBC_XCS0                (Error_using_CPLD)
#define PBC_LCD_EN_B            (Error_using_CPLD)
#define PBC_CODE_B              (Error_using_CPLD)
#define PBC_PSRAM_B             (Error_using_CPLD)
#define PBC_BSTAT1_NF_DET       (Error_using_CPLD)
#define PBC_BSTAT1_KP_ON        (Error_using_CPLD)
#define PBC_BSTAT1_LS           (Error_using_CPLD)
#define PBC_BSTAT1_ATA_IOCS16   (Error_using_CPLD)
#define PBC_BSTAT1_ATA_CBLID    (Error_using_CPLD)
#define PBC_BSTAT1_ATA_DASP     (Error_using_CPLD)
#define PBC_BSTAT1_PWR_RDY      (Error_using_CPLD)
#define PBC_BSTAT1_SD1_WP       (Error_using_CPLD)
#define PBC_BSTAT1_SD2_WP       (Error_using_CPLD)
#define PBC_BSTAT1_FS1          (Error_using_CPLD)
#define PBC_BSTAT1_FS2          (Error_using_CPLD)
#define PBC_BSTAT1_PTT          (Error_using_CPLD)
#define PBC_BSTAT1_MC13783_IN   (Error_using_CPLD)
#define PBC_BCTRL1_ERST         (Error_using_CPLD)
#define PBC_BCTRL1_URST         (Error_using_CPLD)
#define PBC_BCTRL1_UENA         (Error_using_CPLD)
#define PBC_BCTRL1_UENB         (Error_using_CPLD)
#define PBC_BCTRL1_UENCE        (Error_using_CPLD)
#define PBC_BCTRL1_IREN         (Error_using_CPLD)
#define PBC_BCTRL1_LED0         (Error_using_CPLD)
#define PBC_BCTRL1_LED1         (Error_using_CPLD)
#define PBC_BCTRL1_SENSOR1_ON	(Error_using_CPLD)
#define PBC_BCTRL1_SENSOR2_ON	(Error_using_CPLD)
#define PBC_BCTRL1_BEND         (Error_using_CPLD)
#define PBC_BCTRL1_LCDON        (Error_using_CPLD)
#define PBC_BCTRL2_USELA	(Error_using_CPLD)
#define PBC_BCTRL2_USELB	(Error_using_CPLD)
#define PBC_BCTRL2_USELC	(Error_using_CPLD)
#define PBC_BCTRL2_UMODENA	(Error_using_CPLD)
#define PBC_BCTRL2_UMODENC	(Error_using_CPLD)
#define PBC_BCTRL2_CSI_EN	(Error_using_CPLD)
#define PBC_BCTRL2_ATA_EN	(Error_using_CPLD)
#define PBC_BCTRL2_ATA_SEL	(Error_using_CPLD)
#define PBC_BCTRL2_IRDA_MOD	(Error_using_CPLD)
#define PBC_BCTRL2_LDC_RST0	(Error_using_CPLD)
#define PBC_BCTRL2_LDC_RST1	(Error_using_CPLD)
#define PBC_BCTRL2_LDC_RST2	(Error_using_CPLD)
#define PBC_BCTRL2_LDCIO_EN	(Error_using_CPLD)
#define PBC_BCTRL2_CT_CS	(Error_using_CPLD)
#define PBC_BCTRL2_VPP_EN	(Error_using_CPLD)
#define PBC_BCTRL2_VCC_EN	(Error_using_CPLD)
#define PBC_BCTRL3_OTG_FS_SEL	(Error_using_CPLD)
#define PBC_BCTRL3_OTG_FS_EN	(Error_using_CPLD)
#define PBC_BCTRL3_FSH_SEL	(Error_using_CPLD)
#define PBC_BCTRL3_FSH_EN	(Error_using_CPLD)
#define PBC_BCTRL3_HSH_SEL	(Error_using_CPLD)
#define PBC_BCTRL3_HSH_EN	(Error_using_CPLD)
#define PBC_BCTRL3_FSH_MOD	(Error_using_CPLD)
#define PBC_BCTRL3_OTG_HS_EN	(Error_using_CPLD)
#define PBC_BCTRL3_OTG_VBUS_EN	(Error_using_CPLD)
#define PBC_BCTRL3_FSH_VBUS_EN	(Error_using_CPLD)
#define PBC_BCTRL3_CARD1_SEL	(Error_using_CPLD)
#define PBC_BCTRL3_CARD2_SEL	(Error_using_CPLD)
#define PBC_BCTRL3_SYNTH_RST	(Error_using_CPLD)
#define PBC_BCTRL3_VSIM_EN	(Error_using_CPLD)
#define PBC_BCTRL3_VESIM_EN	(Error_using_CPLD)
#define PBC_BCTRL3_SPI3_RESET	(Error_using_CPLD)
#define PBC_BCTRL4_CSI_MSB_EN	(Error_using_CPLD)
#define PBC_BCTRL4_REGEN_SEL	(Error_using_CPLD)
#define PBC_BCTRL4_USER_OFF	(Error_using_CPLD)
#define PBC_BCTRL4_VIB_EN	(Error_using_CPLD)
#define PBC_BCTRL4_PCMCIA_EN	(Error_using_CPLD)
#define PBC_INT_CS8900A         (Error_using_CPLD)
#define PBC_INTSTATUS_REG	(Error_using_CPLD)
#define PBC_INTCURR_STATUS_REG	(Error_using_CPLD)
#define PBC_INTMASK_SET_REG	(Error_using_CPLD)
#define PBC_INTMASK_CLEAR_REG	(Error_using_CPLD)
#define EXPIO_PARENT_INT	(Error_using_CPLD)
#define EXPIO_INT_LOW_BAT	(Error_using_CPLD)
#define EXPIO_INT_PB_IRQ	(Error_using_CPLD)
#define EXPIO_INT_OTG_FS_OVR	(Error_using_CPLD)
#define EXPIO_INT_FSH_OVR	(Error_using_CPLD)
#define EXPIO_INT_RES4		(Error_using_CPLD)
#define EXPIO_INT_RES5		(Error_using_CPLD)
#define EXPIO_INT_RES6		(Error_using_CPLD)
#define EXPIO_INT_RES7		(Error_using_CPLD)
#define EXPIO_INT_ENET_INT	(Error_using_CPLD)
#define EXPIO_INT_OTG_FS_INT	(Error_using_CPLD)
#define EXPIO_INT_XUART_INTA	(Error_using_CPLD)
#define EXPIO_INT_XUART_INTB	(Error_using_CPLD)
#define EXPIO_INT_SYNTH_IRQ	(Error_using_CPLD)
#define EXPIO_INT_CE_INT1	(Error_using_CPLD)
#define EXPIO_INT_CE_INT2	(Error_using_CPLD)
#define EXPIO_INT_RES15		(Error_using_CPLD)
/*#define MXC_MAX_EXP_IO_LINES	(Error_using_CPLD)*/
#define CS8900AIRQ              (Error_using_CPLD)
#define CS8900A_BASE_ADDRESS    (Error_using_CPLD)
/*#define MXC_PMIC_INT_LINE	(Error_using_CPLD)*/

#define AHB_FREQ                133000000
#define IPG_FREQ                66500000

#define PBC_ATA_SIGNAL_ACTIVE()   (Error_using_CPLD)
#define PBC_ATA_SIGNAL_INACTIVE() (Error_using_CPLD)

#endif				/* __ASM_ARCH_MXC_BOARD_MX31ADS_H__ */
